﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace _0216ADMSample
{
    public partial class DevForm : Form
    {
        private short DevId; //デバイスID

        private int SaveLeng; //ファイルへの保存制限数
        private TextWriter SaveFile; //保存ファイルストリーム
        private int TotalLeng; //取り込み済みデータ長
        private Boolean StopReq; //サンプリング停止要求
        private int[,] SamplingBuffer; //取り込みバッファ
        private byte ChSel; //取り込みチャンネル選択 0:ch1 1:ch2 2:ch1&ch2

        public DevForm()
        {
            InitializeComponent();
        }

        public void SetId(short Id)
        {
            byte[] Data = new byte[2];

            DevId = Id;

            //トリガ選択コンボボックスを初期化
            TriggerTypeSel.SelectedIndex = 0;

            //トリガチャンネル選択コンボボックスを初期化
            TriggerCh.SelectedIndex = 0;

            //イネーブルディセーブル制御
            ControlEnable(true);
            StatusView.Text = "オープンされました";

            TriggerTypeChange();
            ClkSel_Change();
            RangeSel_Change();

            this.Text = "ID = " + DevId.ToString() + "  Panel";

            Timer2.Interval = 300;
            Timer2.Enabled = true;
        }

        private void ControlEnable(Boolean en)
        {
            RangeSel_0.Enabled = en;
            RangeSel_1.Enabled = en;
            ChChk_0.Enabled = en;
            ChChk_1.Enabled = en;
            TriggerTypeSel.Enabled = en;
            TriggerLevel.Enabled = en;
            TriggerCh.Enabled = en;
            ClkSel_0.Enabled = en;
            ClkSel_1.Enabled = en;
            ClockInterval.Enabled = en;
            PreTriggerLen.Enabled = en;
            LimitLeng.Enabled = en;
            StartBtn.Enabled = en;
            StopBtn.Enabled = !en;
            TriggerBtn.Enabled = !en;
        }

        private void DevForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (StopBtn.Enabled) //サンプリングがストップしていない場合は
            {
                Timer1.Enabled = false;
                SmpleStop();
                StatusView.Text = "サンプリング終了\r\nデータ数 = " + TotalLeng.ToString();
            }
        }

        private void StartBtn_Click(object sender, EventArgs e)
        {
            string tmpPath;
            short retErr;
            byte TrgType;
            int TrgLevel = 1;
            byte TrgCh;
            short HysLevel;
            byte ClockSel;
            int ClkTime = 500;
            int PreLen;

            //チャンネル選択
            if( ChChk_0.Checked )
            {
                if(ChChk_1.Checked )
                    ChSel = 2;//ch1とch2同時取込
                else
                    ChSel = 0;//ch1のみ取込
            }
            else
            {
                if(ChChk_1.Checked )
                    ChSel = 1;//ch2のみ取込
                else
                {
                    MessageBox.Show("チャンネルが選択されていません");
                    return;
                }
            }

            //トリガ設定
            TrgType = (byte) TriggerTypeSel.SelectedIndex;
            if( TrgType == 2 || TrgType == 3 )
            {
                if( int.TryParse(TriggerLevel.Text, out TrgLevel))
                {
                    if( TrgLevel < 1 || TrgLevel > 65534 )
                    {
                        MessageBox.Show("トリガレベルが不正です");
                        return;
                    }
                }
                else
                {
                    MessageBox.Show("トリガレベル入力値が不正です");
                    return;
                }
            }

            HysLevel = 100;
            TrgCh = (byte) TriggerCh.SelectedIndex;

            //クロック設定
            if (ClkSel_0.Checked)
            {
                ClockSel = 0; //内部クロック
                if (int.TryParse(ClockInterval.Text, out ClkTime))
                {
                    if (ClkTime > 2147483647 || ClkTime < 500)
                    {
                        MessageBox.Show("クロック間隔設定値が不正です");
                        return;
                    }
                }
                else
                {
                    MessageBox.Show("クロック間隔入力値が不正です");
                    return;
                }
            }
            else
            {
                ClockSel = 1; //外部クロック
            }

            //プレトリガ長
            if( int.TryParse(PreTriggerLen.Text, out PreLen) )
            {
                if( PreLen > 262144 || PreLen < 0 )
                {
                    MessageBox.Show("プレトリガ設定値が不正です");
                    return;
                }
            }
            else
            {
                MessageBox.Show("プレトリガ入力値が不正です");
                return;
            }


            //ファイルへの最大保存数
            if( int.TryParse(LimitLeng.Text, out SaveLeng) )
            {
                if( SaveLeng < 0 || SaveLeng > 100000000 )
                {
                    MessageBox.Show("保存数の設定値が不正です");
                    return;
                }
            }
            else
            {
                MessageBox.Show("保存数の入力値が不正です");
                return;
            }
            
            SamplingBuffer = new int[2,SaveLeng];


            //保存先ファイル名の取得
            SaveFileDialog1.Filter = "データファイル(*.dat)|*.dat";
            if( SaveFileDialog1.ShowDialog() != DialogResult.OK )
                return;

            tmpPath = SaveFileDialog1.FileName;

            if (File.Exists(tmpPath)) //既存ファイルがある場合は消去
                File.Delete(tmpPath);

            SaveFile = new StreamWriter(File.OpenWrite(tmpPath)); //ファイルを開く

            //クロック設定
            retErr = TUSB16AD.TUSB0216AD_AdClk_Set(DevId, ClkTime, ClockSel);
            if( retErr != 0 )
            {
                SaveFile.Close();
                MessageBox.Show(TUSB16AD.GetErrMessage(retErr));
                return;
            }

            //トリガレベル設定
            if( TrgType == 2 || TrgType == 3 )
            {
                retErr = TUSB16AD.TUSB0216AD_Level_Set(DevId, TrgLevel, HysLevel);
                if( retErr != 0 )
                {
                    SaveFile.Close();
                    MessageBox.Show(TUSB16AD.GetErrMessage(retErr));
                    return;
                }
            }

            //測定開始
            retErr = TUSB16AD.TUSB0216AD_Start(DevId, ChSel, PreLen, TrgType, TrgCh);
            if( retErr == 0 ) //正常にスタートできた
            {
                StopReq = false;
                TotalLeng = 0;
                ControlEnable(false);
                Timer1.Interval = 50;
                Timer1.Enabled = true;
            }
            else //正常にスタートできなかった
            {
                SaveFile.Close();
                MessageBox.Show(TUSB16AD.GetErrMessage(retErr));
            }
        }

        //取り込み停止ボタンの処理
        private void StopBtn_Click(object sender, EventArgs e)
        {
            StopReq = true;
        }

        //タイマー処理(サンプリング中のみ)
        private void Timer1_Tick(object sender, EventArgs e)
        {
            short retErr;
            byte Status;
            byte[] ovf = new byte[2];
            int[] leng = new int[2];
            int tmpleng;
            int buflen;
            int[] tmpData;

            if( StopReq ) //停止要求の確認
            {
                SmpleStop();
                StatusView.Text = "サンプリング終了\r\nデータ数 = " + TotalLeng.ToString();
                return;
            }

            //バッファステータスの確認
            retErr = TUSB16AD.TUSB0216AD_Ad_Status(DevId, out Status, ovf, leng);

            if( retErr != 0 )
            {
                MessageBox.Show(TUSB16AD.GetErrMessage(retErr));
                SmpleStop();
                return;
            }

            switch( Status )
            {
                case 1: //トリガ待ち
                    StatusView.Text = "トリガ待ちです";
                    break;
                case 3: //サンプリング中
                    switch( ChSel )
                    {
                        case 0: //ch1
                            if(ovf[0] != 0)
                            {
                                MessageBox.Show("バッファオーバーフローです");
                                SmpleStop();
                                return;
                            }
                            tmpleng = leng[0];
                            break;
                        case 1: //ch2
                            if( ovf[1] != 0 )
                            {
                                MessageBox.Show("バッファオーバーフローです");
                                SmpleStop();
                                return;
                            }
                            tmpleng = leng[1];
                            break;
                        case 2: //ch1&ch2
                            if( (ovf[0] != 0) || (ovf[1] != 0) )
                            {
                                MessageBox.Show("バッファオーバーフローです");
                                SmpleStop();
                                return;
                            }
                            if( leng[0] < leng[1])
                                tmpleng = leng[0];
                            else
                                tmpleng = leng[1];
                            break;
                        default:
                            tmpleng = 0;
                            break;
                    }

                    if( (SaveLeng != 0) && ((TotalLeng + tmpleng) >= SaveLeng) ) //最大取り込み長を超えていたら
                    {
                        tmpleng = SaveLeng - TotalLeng;
                        buflen = tmpleng;
                        tmpData = new int[buflen];//データ取り込み用バッファの確保
                        switch( ChSel )
                        {
                            case 0://ch1のみ
                                retErr = TUSB16AD.TUSB0216AD_Ad_Data(DevId, 0, tmpData, out tmpleng); //ch1データの読み込み
                                CopyData(0, tmpData, tmpleng);
                                break;
                            case 1: //ch2のみ
                                retErr = TUSB16AD.TUSB0216AD_Ad_Data(DevId, 1, tmpData, out tmpleng); //ch2データの読み込み
                                CopyData(1, tmpData, tmpleng);
                                break;
                            case 2: //ch1&ch2
                                retErr = TUSB16AD.TUSB0216AD_Ad_Data(DevId, 0, tmpData, out tmpleng); //ch1データの読み込み
                                CopyData(0, tmpData, tmpleng);
                                retErr = TUSB16AD.TUSB0216AD_Ad_Data(DevId, 1, tmpData, out tmpleng); //ch2データの読み込み
                                CopyData(1, tmpData, tmpleng);
                                break;
                        }
                        TotalLeng = TotalLeng + tmpleng;
                        SmpleStop();
                        StatusView.Text = "サンプリング終了\r\nデータ数 = " + TotalLeng.ToString();
                    }
                    else
                    {
                        buflen = tmpleng;
                        tmpData = new int[buflen]; //データ取り込み用バッファの確保
                        switch(ChSel)
                        {
                            case 0: //ch1のみ
                                retErr = TUSB16AD.TUSB0216AD_Ad_Data(DevId, 0, tmpData, out tmpleng); //ch1データの読み込み
                                CopyData(0, tmpData, tmpleng);
                                break;
                            case 1: //ch2のみ;
                                retErr = TUSB16AD.TUSB0216AD_Ad_Data(DevId, 1, tmpData, out tmpleng); //ch2データの読み込み
                                CopyData(1, tmpData, tmpleng);
                                break;
                            case 2: //ch1&ch2
                                retErr = TUSB16AD.TUSB0216AD_Ad_Data(DevId, 0, tmpData, out tmpleng); //ch1データの読み込み
                                CopyData(0, tmpData, tmpleng);
                                retErr = TUSB16AD.TUSB0216AD_Ad_Data(DevId, 1, tmpData, out tmpleng); //ch2データの読み込み
                                CopyData(1, tmpData, tmpleng);
                                break;
                        }
                        TotalLeng = TotalLeng + tmpleng;
                        StatusView.Text = "サンプリング中です\r\nデータ数 = " + TotalLeng.ToString();
                    }
                    break;
                default: //停止中
                    StatusView.Text = "停止中";
                    break;
            }
        }

        //データバッファへのコピー処理
        private void CopyData(byte Ch,int[] Data, int leng)
        {
            int cnt;
            int pointer;

            pointer = 0;
            for( cnt = TotalLeng ; cnt < ( TotalLeng + leng ) ; cnt++ )
            {
                SamplingBuffer[Ch, cnt] = Data[pointer];
                pointer = pointer + 1;
            }
        }
        //データの書き込み処理
        private void WriteData()
        {
            int cnt;

            for(cnt = 0 ; cnt < TotalLeng ; cnt++ )
            {
                switch( ChSel)
                {
                    case 0: //ch1 のみ
                        SaveFile.WriteLine(SamplingBuffer[0, cnt].ToString());
                        break;
                    case 1: //ch2 のみ
                        SaveFile.WriteLine(SamplingBuffer[1, cnt].ToString());
                        break;
                    case 2: //両ch
                        SaveFile.WriteLine(SamplingBuffer[0, cnt].ToString() + "," + SamplingBuffer[1, cnt].ToString());
                        break;
                }
            }
        }

        //連続取り込み停止時の処理
        private void SmpleStop()
        {
            Timer1.Enabled = false;
            TUSB16AD.TUSB0216AD_Stop(DevId); //サンプリング停止
            StatusView.Text = "データ保存中です";
            Refresh();
            WriteData();
            SaveFile.Close(); //ファイルを閉じる
            ControlEnable(true);
        }

        private byte GetDoButton()
        {
            byte bitdata = 0;

            if (DO0.Checked)
                bitdata += 0x1;
            if (DO1.Checked)
                bitdata += 0x2;
            if (DO2.Checked)
                bitdata += 0x4;
            if (DO3.Checked)
                bitdata += 0x8;
            if (DO4.Checked)
                bitdata += 0x10;
            if (DO5.Checked)
                bitdata += 0x20;
            return bitdata;
        }

        private System.Drawing.Color DispCol(Boolean OnOff)
        {
            if( OnOff )
                return System.Drawing.Color.Red;
            else
                return System.Drawing.Color.Gray;
        }

        private void DispDin(byte Data)
        {
            DI0.BackColor = DispCol((Data & 0x1) != 0);
            DI1.BackColor = DispCol((Data & 0x2) != 0);
            DI2.BackColor = DispCol((Data & 0x4) != 0);
            DI3.BackColor = DispCol((Data & 0x8) != 0);
            DI4.BackColor = DispCol((Data & 0x10) != 0);
            DI5.BackColor = DispCol((Data & 0x20) != 0);
        }

        private void Timer2_Tick(object sender, EventArgs e)
        {
            short retErr;
            byte OutData;
            byte InData;

            OutData = GetDoButton();
            retErr = TUSB16AD.TUSB0216AD_DIO_Out(DevId, OutData);

            retErr = TUSB16AD.TUSB0216AD_DIO_In(DevId, out InData);
            DispDin(InData);
        }

        //トリガボタンの処理
        private void TriggerBtn_Click(object sender, EventArgs e)
        {
            short retErr;
            retErr = TUSB16AD.TUSB0216AD_Trigger(DevId); //トリガコマンド
            if (retErr != 0)
            {
                MessageBox.Show(TUSB16AD.GetErrMessage(retErr));
                StopReq = true;
            }
        }

        //'トリガタイプの選択
        private void TriggerTypeChange()
        {
            switch(TriggerTypeSel.SelectedIndex)
            {
                case 0: //ソフトトリガ
                    TriggerBtn.Visible = true;
                    TriggerLevel.Visible = false;
                    TriggerCh.Visible = false;
                    break;
                case 1: //外部ディジタルトリガ
                    TriggerBtn.Visible = false;
                    TriggerLevel.Visible = false;
                    TriggerCh.Visible = false;
                    break;
                default: //その他
                    TriggerBtn.Visible = false;
                    TriggerLevel.Visible = true;
                    TriggerCh.Visible = true;
                    break;
            }
        }

        private void TriggerTypeSel_SelectedIndexChanged(object sender, EventArgs e)
        {
            TriggerTypeChange();
        }

        //クロック選択ラジオボタン処理
        private void ClkSel_Change()
        {
            if( ClkSel_0.Checked )
                ClockInterval.Enabled = true; //内部クロック入力を有効にする
            else
                ClockInterval.Enabled = false;
        }

        private void ClkSel_CheckedChanged(object sender, EventArgs e)
        {
            ClkSel_Change();
        }
        
        private void RangeSel_Change()
        {
            short retErr;
            byte Range1;
            byte Range2;

            if(RangeSel_0.Enabled && RangeSel_1.Enabled)
            {
                if(RangeSel_0.SelectedIndex < 0)
                    RangeSel_0.SelectedIndex = 0;

                if (RangeSel_1.SelectedIndex < 0)
                    RangeSel_1.SelectedIndex = 0;

                Range1 = (byte) RangeSel_0.SelectedIndex;
                Range2 = (byte) RangeSel_1.SelectedIndex;

                retErr = TUSB16AD.TUSB0216AD_Input_Set(DevId, Range1, Range2);

                if( retErr != 0 )
                    MessageBox.Show(TUSB16AD.GetErrMessage(retErr));
            }
        }

        private void RangeSel_SelectedIndexChanged(object sender, EventArgs e)
        {
            RangeSel_Change();
        }
    }
}
