﻿Imports System.IO

Public Class DevForm
    Private DevId As Short 'デバイスID

    Private SaveLeng As Integer 'ファイルへの保存制限数
    Private SaveFile As TextWriter '保存ファイルストリーム
    Private TotalLeng As Integer '取り込み済みデータ長
    Private StopReq As Boolean 'サンプリング停止要求
    Private SamplingBuffer(,) As Integer '取り込みバッファ
    Private ChSel As Byte '取り込みチャンネル選択 0:ch1 1:ch2 2:ch1&ch2

    Public Sub SetId(ByVal Id As Short)
        Dim Data(2) As Byte

        DevId = Id

        'トリガ選択コンボボックスを初期化
        TriggerTypeSel.SelectedIndex = 0

        'トリガチャンネル選択コンボボックスを初期化
        TriggerCh.SelectedIndex = 0

        'イネーブルディセーブル制御
        ControlEnable(True)
        StatusView.Text = "オープンされました"

        TriggerTypeChange()
        ClkSel_Change()
        RangeSel_Change()

        Me.Text = "ID = " + DevId.ToString() + "  Panel"

        Timer2.Interval = 300
        Timer2.Enabled = True

    End Sub

    'イネーブルディセーブル制御(画面の表示コントロールの状態)
    Private Sub ControlEnable(ByVal en As Boolean)
        RangeSel_0.Enabled = en
        RangeSel_1.Enabled = en
        ChChk_0.Enabled = en
        ChChk_1.Enabled = en
        TriggerTypeSel.Enabled = en
        TriggerLevel.Enabled = en
        TriggerCh.Enabled = en
        ClkSel_0.Enabled = en
        ClkSel_1.Enabled = en
        ClockInterval.Enabled = en
        PreTriggerLen.Enabled = en
        LimitLeng.Enabled = en
        StartBtn.Enabled = en
        StopBtn.Enabled = Not en
        TriggerBtn.Enabled = Not en
    End Sub

    Private Sub DevForm_FormClosed(sender As System.Object, e As System.Windows.Forms.FormClosedEventArgs) Handles MyBase.FormClosed
        If StopBtn.Enabled Then 'サンプリングがストップしていない場合は
            Timer1.Enabled = False
            SmpleStop()
            StatusView.Text = "サンプリング終了" + vbNewLine + "データ数 = " + TotalLeng.ToString()
        End If
    End Sub

    '取り込み開始ボタンの処理
    Private Sub StartBtn_Click(sender As System.Object, e As System.EventArgs) Handles StartBtn.Click
        Dim tmpPath As String
        Dim retErr As Short
        Dim TrgType As Byte
        Dim TrgLevel As Integer = 1
        Dim TrgCh As Byte
        Dim HysLevel As Short
        Dim ClockSel As Byte
        Dim ClkTime As Integer = 500
        Dim PreLen As Integer

        'チャンネル選択
        If ChChk_0.Checked Then
            If ChChk_1.Checked Then
                ChSel = 2 'ch1とch2同時取込
            Else
                ChSel = 0 'ch1のみ取込
            End If
        Else
            If ChChk_1.Checked Then
                ChSel = 1 'ch2のみ取込
            Else
                MessageBox.Show("チャンネルが選択されていません")
                Return
            End If
        End If

        'トリガ設定
        TrgType = TriggerTypeSel.SelectedIndex
        If TrgType = 2 Or TrgType = 3 Then
            If Integer.TryParse(TriggerLevel.Text, TrgLevel) Then
                If TrgLevel < 1 Or TrgLevel > 65534 Then
                    MessageBox.Show("トリガレベルが不正です")
                    Return
                End If
            Else
                MessageBox.Show("トリガレベル入力値が不正です")
                Return
            End If
        End If
        HysLevel = 100
        TrgCh = TriggerCh.SelectedIndex

        'クロック設定
        If ClkSel_0.Checked Then
            ClockSel = 0 '内部クロック
            If Integer.TryParse(ClockInterval.Text, ClkTime) Then
                If ClkTime > 2147483647 Or ClkTime < 500 Then
                    MessageBox.Show("クロック間隔設定値が不正です")
                    Return
                End If
            Else
                MessageBox.Show("クロック間隔入力値が不正です")
                Return
            End If
        Else
            ClockSel = 1 '外部クロック
        End If

        'プレトリガ長
        If Integer.TryParse(PreTriggerLen.Text, PreLen) Then
            If PreLen > 262144 Or PreLen < 0 Then
                MessageBox.Show("プレトリガ設定値が不正です")
                Return
            End If
        Else
            MessageBox.Show("プレトリガ入力値が不正です")
            Return
        End If

        'ファイルへの最大保存数
        If Integer.TryParse(LimitLeng.Text, SaveLeng) Then
            If SaveLeng < 0 Or SaveLeng > 100000000 Then
                MessageBox.Show("保存数の設定値が不正です")
                Return
            End If
        Else
            MessageBox.Show("保存数の入力値が不正です")
            Return
        End If
        ReDim SamplingBuffer(1, SaveLeng)

        '保存先ファイル名の取得
        SaveFileDialog1.Filter = "データファイル(*.dat)|*.dat"
        If SaveFileDialog1.ShowDialog() <> DialogResult.OK Then
            Return
        End If

        tmpPath = SaveFileDialog1.FileName

        If File.Exists(tmpPath) Then '既存ファイルがある場合は消去
            File.Delete(tmpPath)
        End If

        SaveFile = New StreamWriter(File.OpenWrite(tmpPath)) 'ファイルを開く

        'クロック設定
        retErr = TUSB0216AD_AdClk_Set(DevId, ClkTime, ClockSel)
        If retErr <> 0 Then
            SaveFile.Close()
            TUSB0216AD_ErrorDisp(retErr)
            Return
        End If

        'トリガレベル設定
        If TrgType = 2 Or TrgType = 3 Then
            retErr = TUSB0216AD_Level_Set(DevId, TrgLevel, HysLevel)
            If retErr <> 0 Then
                SaveFile.Close()
                TUSB0216AD_ErrorDisp(retErr)
                Return
            End If
        End If

        '測定開始
        retErr = TUSB0216AD_Start(DevId, ChSel, PreLen, TrgType, TrgCh)
        If retErr = 0 Then '正常にスタートできた
            StopReq = False
            TotalLeng = 0
            ControlEnable(False)
            Timer1.Interval = 50
            Timer1.Enabled = True
        Else '正常にスタートできなかった
            SaveFile.Close()
            TUSB0216AD_ErrorDisp(retErr)
        End If
    End Sub

    '取り込み停止ボタンの処理
    Private Sub StopBtn_Click(sender As System.Object, e As System.EventArgs) Handles StopBtn.Click
        StopReq = True
    End Sub

    'タイマー処理(サンプリング中のみ)
    Private Sub Timer1_Tick(sender As System.Object, e As System.EventArgs) Handles Timer1.Tick
        Dim retErr As Short
        Dim Status As Byte
        Dim ovf(1) As Byte
        Dim leng(1) As Integer
        Dim tmpleng As Integer
        Dim buflen As Integer
        Dim tmpData() As Integer

        If StopReq Then '停止要求の確認
            SmpleStop()
            StatusView.Text = "サンプリング終了" + vbNewLine + "データ数 = " + TotalLeng.ToString()
            Return
        End If

        'バッファステータスの確認
        retErr = TUSB0216AD_Ad_Status(DevId, Status, ovf, leng)

        If retErr <> 0 Then
            TUSB0216AD_ErrorDisp(retErr)
            SmpleStop()
            Return
        End If

        Select Case Status
            Case 1 'トリガ待ち
                StatusView.Text = "トリガ待ちです"
            Case 3 'サンプリング中

                Select Case ChSel
                    Case 0 'ch1
                        If ovf(0) Then
                            MessageBox.Show("バッファオーバーフローです")
                            SmpleStop()
                            Return
                        End If
                        tmpleng = leng(0)
                    Case 1 'ch2
                        If ovf(1) Then
                            MessageBox.Show("バッファオーバーフローです")
                            SmpleStop()
                            Return
                        End If
                        tmpleng = leng(1)
                    Case 2 'ch1&ch2
                        If ovf(0) Or ovf(1) Then
                            MessageBox.Show("バッファオーバーフローです")
                            SmpleStop()
                            Return
                        End If
                        If leng(0) < leng(1) Then
                            tmpleng = leng(0)
                        Else
                            tmpleng = leng(1)
                        End If
                End Select

                If (SaveLeng <> 0) And ((TotalLeng + tmpleng) >= SaveLeng) Then '最大取り込み長を超えていたら
                    tmpleng = SaveLeng - TotalLeng
                    buflen = tmpleng
                    ReDim tmpData(buflen) 'データ取り込み用バッファの確保
                    Select Case ChSel
                        Case 0 'ch1のみ
                            retErr = TUSB0216AD_Ad_Data(DevId, 0, tmpData, tmpleng) 'ch1データの読み込み
                            CopyData(0, tmpData, tmpleng)
                        Case 1 'ch2のみ
                            retErr = TUSB0216AD_Ad_Data(DevId, 1, tmpData, tmpleng) 'ch2データの読み込み
                            CopyData(1, tmpData, tmpleng)
                        Case 2 'ch1&ch2
                            retErr = TUSB0216AD_Ad_Data(DevId, 0, tmpData, tmpleng) 'ch1データの読み込み
                            CopyData(0, tmpData, tmpleng)
                            retErr = TUSB0216AD_Ad_Data(DevId, 1, tmpData, tmpleng) 'ch2データの読み込み
                            CopyData(1, tmpData, tmpleng)
                    End Select
                    TotalLeng = TotalLeng + tmpleng
                    SmpleStop()
                    StatusView.Text = "サンプリング終了" + vbNewLine + "データ数 = " + TotalLeng.ToString()
                Else
                    buflen = tmpleng
                    ReDim tmpData(buflen) 'データ取り込み用バッファの確保
                    Select Case ChSel
                        Case 0 'ch1のみ
                            retErr = TUSB0216AD_Ad_Data(DevId, 0, tmpData, tmpleng) 'ch1データの読み込み
                            CopyData(0, tmpData, tmpleng)
                        Case 1 'ch2のみ
                            retErr = TUSB0216AD_Ad_Data(DevId, 1, tmpData, tmpleng) 'ch2データの読み込み
                            CopyData(1, tmpData, tmpleng)
                        Case 2 'ch1&ch2
                            retErr = TUSB0216AD_Ad_Data(DevId, 0, tmpData, tmpleng) 'ch1データの読み込み
                            CopyData(0, tmpData, tmpleng)
                            retErr = TUSB0216AD_Ad_Data(DevId, 1, tmpData, tmpleng) 'ch2データの読み込み
                            CopyData(1, tmpData, tmpleng)
                    End Select
                    TotalLeng = TotalLeng + tmpleng
                    StatusView.Text = "サンプリング中です" & vbCrLf & "データ数 = " & CStr(TotalLeng)
                End If
            Case Else '停止中
                StatusView.Text = "停止中"
        End Select
    End Sub

    'データバッファへのコピー処理
    Private Sub CopyData(ByVal Ch As Byte, ByVal Data() As Integer, ByVal leng As Integer)
        Dim cnt As Integer
        Dim pointer As Integer

        pointer = 0
        For cnt = TotalLeng To TotalLeng + leng - 1
            SamplingBuffer(Ch, cnt) = Data(pointer)
            pointer = pointer + 1
        Next cnt
    End Sub

    'データの書き込み処理
    Private Sub WriteData()
        Dim cnt As Integer
        For cnt = 0 To TotalLeng - 1
            Select Case ChSel
                Case 0 'ch1 のみ
                    SaveFile.WriteLine(SamplingBuffer(0, cnt).ToString)
                Case 1 'ch2 のみ
                    SaveFile.WriteLine(SamplingBuffer(1, cnt).ToString)
                Case 2 '両ch
                    SaveFile.WriteLine(SamplingBuffer(0, cnt).ToString + "," + SamplingBuffer(1, cnt).ToString)
            End Select
        Next cnt
    End Sub

    '連続取り込み停止時の処理
    Private Sub SmpleStop()
        Timer1.Enabled = False
        TUSB0216AD_Stop(DevId) 'サンプリング停止
        StatusView.Text = "データ保存中です"
        Refresh()
        WriteData()
        SaveFile.Close() 'ファイルを閉じる
        ControlEnable(True)
    End Sub

    Private Function GetDoButton() As Byte
        Dim bitdata As Byte = 0
        If DO0.CheckState Then
            bitdata += &H1S
        End If
        If DO1.CheckState Then
            bitdata += &H2S
        End If
        If DO2.CheckState Then
            bitdata += &H4S
        End If
        If DO3.CheckState Then
            bitdata += &H8S
        End If
        If DO4.CheckState Then
            bitdata += &H10S
        End If
        If DO5.CheckState Then
            bitdata += &H20S
        End If
        Return bitdata
    End Function

    Private Function DispCol(ByRef OnOff As Boolean) As System.Drawing.Color
        If OnOff Then
            Return System.Drawing.Color.Red
        Else
            Return System.Drawing.Color.Gray
        End If
    End Function

    Private Sub DispDin(ByVal Data As Byte)
        DI0.BackColor = DispCol((Data And &H1S) <> 0)
        DI1.BackColor = DispCol((Data And &H2S) <> 0)
        DI2.BackColor = DispCol((Data And &H4S) <> 0)
        DI3.BackColor = DispCol((Data And &H8S) <> 0)
        DI4.BackColor = DispCol((Data And &H10S) <> 0)
        DI5.BackColor = DispCol((Data And &H20S) <> 0)
    End Sub

    Private Sub Timer2_Tick(sender As System.Object, e As System.EventArgs) Handles Timer2.Tick
        Dim retErr As Short
        Dim OutData As Byte
        Dim InData As Byte

        OutData = GetDoButton()
        retErr = TUSB0216AD_DIO_Out(DevId, OutData)

        retErr = TUSB0216AD_DIO_In(DevId, InData)
        DispDin(InData)
    End Sub

    'トリガボタンの処理
    Private Sub TriggerBtn_Click(sender As System.Object, e As System.EventArgs) Handles TriggerBtn.Click
        Dim retErr As Short
        retErr = TUSB0216AD_Trigger(DevId) 'トリガコマンド
        If retErr <> 0 Then
            TUSB0216AD_ErrorDisp(retErr)
            StopReq = True
        End If
    End Sub

    'トリガタイプの選択
    Private Sub TriggerTypeChange()
        Select Case TriggerTypeSel.SelectedIndex
            Case 0 'ソフトトリガ
                TriggerBtn.Visible = True
                TriggerLevel.Visible = False
                TriggerCh.Visible = False
            Case 1 '外部ディジタルトリガ
                TriggerBtn.Visible = False
                TriggerLevel.Visible = False
                TriggerCh.Visible = False
            Case Else 'アナログトリガ等
                TriggerBtn.Visible = False
                TriggerLevel.Visible = True
                TriggerCh.Visible = True
        End Select
    End Sub

    Private Sub TriggerTypeSel_SelectedIndexChanged(sender As System.Object, e As System.EventArgs) Handles TriggerTypeSel.SelectedIndexChanged
        TriggerTypeChange()
    End Sub

    'クロック選択ラジオボタン処理
    Private Sub ClkSel_Change()
        If ClkSel_0.Checked Then
            ClockInterval.Enabled = True '内部クロック入力を有効にする
        Else
            ClockInterval.Enabled = False
        End If
    End Sub

    Private Sub ClkSel_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles ClkSel_1.CheckedChanged, ClkSel_0.CheckedChanged
        ClkSel_Change()
    End Sub

    Private Sub RangeSel_Change()
        Dim retErr As Short
        Dim Range1 As Byte
        Dim Range2 As Byte

        If ((RangeSel_0.Enabled = True) And (RangeSel_1.Enabled = True)) Then

            If (RangeSel_0.SelectedIndex < 0) Then
                RangeSel_0.SelectedIndex = 0
            End If
            If (RangeSel_1.SelectedIndex < 0) Then
                RangeSel_1.SelectedIndex = 0
            End If
            Range1 = RangeSel_0.SelectedIndex
            Range2 = RangeSel_1.SelectedIndex

            retErr = TUSB0216AD_Input_Set(DevId, Range1, Range2)

            If retErr Then
                TUSB0216AD_ErrorDisp(retErr)
            End If
        End If
    End Sub

    Private Sub RangeSel_SelectedIndexChanged(sender As System.Object, e As System.EventArgs) Handles RangeSel_1.SelectedIndexChanged, RangeSel_0.SelectedIndexChanged
        RangeSel_Change()
    End Sub

End Class